
# q-expansion of newform 867.4.a.j, downloaded from the LMFDB on 14 January 2025.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField, ZZ
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K([c/ZZ(den) for c in num]) for num, den in basis_data]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 867
weight = 4
poly_data = [-2, 0, 1]

# The entries in the following list give a basis for the
# coefficient ring in terms of a root of the defining polynomial above.
# Each line consists of the coefficients of the numerator, and a denominator.
basis_data  = [[[1, 0], 1], [[0, 3], 1]]

hecke_ring_character_values = None
aps_data = [[0, 1], [3, 0], [-3, -4], [4, 4], [-33, 4], [-1, -8], [0, 0], [-13, -16], [99, 4], [-222, 16], [-266, -8], [-44, -64], [-285, 36], [-91, 16], [210, 12], [-150, -124], [222, 96], [-200, 156], [-484, -72], [924, -56], [-434, -12], [-254, 0], [498, 12], [-144, 252], [-512, -52], [720, -200], [209, -48], [-117, -84], [556, -48], [513, 52], [-1153, 352], [2049, 20], [-1014, 252], [-986, 172], [-882, 188], [848, 48], [1859, -64], [-2666, 216], [1743, -28], [561, 748], [-654, -184], [-782, 236], [-486, -620], [358, 840], [-2955, -44], [-560, -300], [-2858, -488], [-1501, 336], [3921, -540], [-1258, 768], [339, 916], [-444, -276], [2536, 996], [1896, 232], [-2292, 196], [-3732, -912], [-2433, -1052], [-169, 240], [-3554, -124], [-276, 1236], [3682, -520], [-1176, 400], [-700, -240], [-1488, -808], [-944, -1516], [-930, 96], [95, 1160], [9538, 300], [-852, 1176], [1829, 1344], [-426, -1048], [-1152, 264], [-4808, 600], [-2734, -720], [-5120, -1372], [5316, -684], [-4140, -80], [3868, -492], [1743, 548], [-5995, 1264], [564, 128], [-5761, -2408], [-3936, -1496], [-4369, -640], [-4772, 2376], [-930, 1052], [-5466, 160], [9605, 840], [3342, -2472], [11024, -368], [-5934, 148], [2439, -3340], [-1538, -2248], [-4830, 1612], [5470, -3636], [-14595, 668], [36, -232], [7539, 532], [-10204, 1344], [-6752, 3576], [-4808, 36], [20130, -536], [-19398, -480], [-6216, -1932], [7336, -2788], [9137, 96], [-6372, -2192], [4446, 540], [-9006, -3788], [-14390, -880], [-9230, -2156], [2027, 4152], [-2154, -1552], [-11054, -1872], [1823, 3920], [5721, -828], [1168, 3036], [-2070, 2364], [-99, -1988], [8142, 92], [19001, 1224], [9238, 196], [3051, -3820], [12633, 2164], [17164, -2104], [12984, -2192], [-11102, -3692], [-16311, -908], [21296, 456], [-3478, -5920], [-12613, 4928], [17184, -2664], [14086, -3616], [-3931, -8184], [-10482, 1696], [-11815, -4272], [25512, 3580], [-1496, 2268], [22272, -2424], [-1749, 8932], [7042, -188], [-14379, -3308], [30988, -2872], [-6351, -6484], [-15130, -1088], [20541, 2476], [-22682, -3144], [5346, 7208], [-30316, 2216], [-16104, 2524], [32578, -300], [-18, -4704], [31727, 72], [-23691, -2116], [1558, 8948], [-24705, -5452], [2567, -9960], [7617, -716], [4346, -9992], [20214, -2624], [-10692, -6808], [9684, -9024], [27029, -5760], [-26850, -5384], [-40926, -840], [38133, -124], [27700, 6540], [28114, 2640], [-1748, -1836], [-7278, -11784], [13074, -8356], [32573, 6792], [-17181, 4588], [20795, 6944], [-11353, -6496], [-14379, 11868], [-11780, -9628], [29463, -4364], [5105, 8904], [14831, 7816], [713, 960], [23532, 384], [-4106, -4260], [1908, -4464], [-22410, 3944], [17334, -7640], [-24662, -5436], [-15847, 264], [22114, 4196], [23652, -1416], [-6854, -128], [-14820, 6616], [-31765, 5888], [36624, -1812], [-40923, 7308], [20298, 4800], [-48398, 5440], [29716, -9504], [5718, -5192], [-1854, -432], [19599, 10748], [-56900, -5220], [-18463, 80], [-24199, -13320], [-22092, -6244], [-9162, 14340], [-352, 13240], [66768, 72], [14715, -10972], [-15340, -4864], [-31118, 1444], [22374, -1204], [1582, 4248], [-30396, 2108], [6477, 9836], [-42068, 7008], [-3631, 10616], [10908, 11332], [70857, -804], [-10128, -10476], [-31351, -136], [-25652, 6192], [-24864, 17968], [-9782, 1964], [36054, 16460], [-11098, 5512], [3453, -7332], [49515, 11764], [-46867, 1712], [45744, 7608], [-29521, 17416], [-9794, -6240], [-60916, 1808], [-19218, 11904], [-23752, -11072], [-59154, 9480], [30040, 10284], [-21561, 3884], [17403, -7684], [132, 5044], [-46137, 15612], [-6172, -19056], [25211, -15624], [64019, 432], [-1131, 1932], [-17181, 17300], [30514, 4300], [100065, -1068], [-57439, 760], [-87300, 2892], [45425, 7752], [-40050, 16632], [23088, -9356], [62656, -1512], [-58422, -12724], [40956, 6196], [10672, -18168], [63604, -1180], [-13227, 18956], [76886, 1144], [-71102, -6936], [62112, 8448], [18124, -10176], [5752, 5144], [65655, -3500], [4583, -13440], [27870, -2804], [-288, 16132], [38800, -13804], [-39186, -4696], [-85232, -10632], [17573, -8872], [88007, 11216], [-38824, -23976], [33518, 9720], [-24973, -15720], [53316, -2716], [-91387, 9376], [57173, 7832], [10656, 3172], [16188, 4080], [-82010, 7716], [-16992, 4352], [-38050, -6888], [-46082, 14940], [-117600, -5164], [42982, -12240], [-42450, -12768], [-57697, 23064], [83808, -11400], [-77283, -14836], [9543, 17492], [-35202, 10956], [-50583, -17692], [-116048, -10008], [-115263, 10900], [41240, 20736], [-12144, 28124], [22179, -27100], [49028, 11288], [-59695, -19896], [44472, 1676], [93124, -8272], [-14661, 27772], [-47780, -12052], [34702, -27864], [53826, -8896], [38500, 5472], [-84876, -14948], [41282, 4672], [-24396, -2168], [59274, -22176], [37509, 28716], [82271, 14592], [31872, 2576], [37367, 17128], [103290, -9736], [28257, 24572], [-93380, -17696], [12420, 8888], [-21242, -13872], [142630, -6612], [-28878, 29276], [101519, -11696], [-133542, 9200], [-155509, 7704], [43414, 16800], [-33230, -16548], [-4560, -1256], [-22149, -8076], [18844, 26532], [120231, 3052], [31642, -5580], [69150, -31272], [106864, -1296], [35031, -15244], [10721, 8184], [-77967, -13124], [-59414, 24820], [-95119, -12840], [-2851, -7248], [58884, 13832], [33501, -2964], [104771, 2448], [-17112, -4240], [-11073, 14860], [-142130, -11388], [102131, 1696], [97611, 22956], [-174, 4000], [-61207, -37560], [-95948, -22072], [-93204, -18160], [-146990, -10092], [20279, -6024], [-47952, 23652], [25020, -15400], [84612, 26672], [159117, 3068], [-82470, -5800], [-40124, 36540], [84591, 1100], [-29400, 10112], [20412, 4496], [93593, -11064], [-137671, -17880], [-16197, -31988], [63524, 15408], [110488, 4636], [40620, -13080], [50860, -7732], [-142107, 16956], [-181296, -10004], [-110464, -12312], [-10040, 14248], [149409, 11452], [-7020, -11688], [10799, 14320], [175302, -2652], [-90007, 10584], [-36663, -15044], [-203520, -2004], [38020, -18380], [16833, 12556], [-41090, 35092], [-2289, -42932], [44453, -22104], [55886, 27704], [-90344, 4092], [-65856, -41304], [9382, -32156], [-143769, 19428], [-98508, 3784], [110315, 33312], [-124494, 10284], [-9596, 53088], [-174016, 14128], [140694, -10492], [63292, 17488], [-39474, -13780], [1072, -1900], [-112080, 18136], [67781, 49416], [-65550, 10064], [21594, 8780], [-73238, -48716], [71942, 28400], [85164, -15712], [105185, 24912], [-9357, -10388], [78388, 35112], [18354, -35280], [-93024, -3900], [52252, -12120], [-59143, -16768], [-132522, 29792], [7467, -16388], [-18950, 65764], [55449, 14532], [-41574, -14268], [122550, 1808], [143806, -29624], [-208623, -12756], [-43638, -2372], [24592, -43424], [-240168, -9060], [54960, 43752], [-11637, -12924], [129701, -20448], [72621, 18452]]
